import os

from cmutils.config_loaders import IniConfigLoader
from peewee import *
import json
from playhouse.shortcuts import ReconnectMixin

store_database_proxy = Proxy()


class ReconnectMySQLDatabase(ReconnectMixin, MySQLDatabase):
  pass

def init_store(host, user, password, database):
  store_db_connection = ReconnectMySQLDatabase(
    host=host,
    user=user,
    password=password,
    database=database
  )
  store_database_proxy.initialize(store_db_connection)


class StoreBaseModel(Model):
  def __str__(self):
    r = {}
    for k in self._data.keys():
      try:
        r[k] = str(getattr(self, k))
      except:
        r[k] = json.dumps(getattr(self, k))

    return str(r)

  class Meta:
    database = store_database_proxy


class Store(StoreBaseModel):
  id = IntegerField()
  name = CharField()
  code = CharField()
  platfrom = CharField()
  api_credential = CharField()
  url = CharField()
  service_account_data = TextField()

  class Meta:
    db_table = 'stores'


class StoreSeller(StoreBaseModel):
  id = IntegerField()
  name = CharField()
  slug = CharField()
  state = CharField()
  email = CharField()
  code = CharField()
  gsheet_id = CharField()

  class Meta:
    db_table = 'store_sellers'


class StoreProduct(StoreBaseModel):
  id = BigIntegerField()
  store_seller_id = IntegerField()
  s_product_id = BigIntegerField()
  source = CharField()
  source_product_id = CharField()
  handle = CharField()
  title = CharField()
  description = TextField()
  sku = CharField()
  has_image = BooleanField()

  class Meta:
    db_table = 'store_products'


class GmcStore(StoreBaseModel):
  id = IntegerField()
  merchant_id = CharField()
  service_account_data = CharField()
  active = BooleanField()

  class Meta:
    db_table = 'gmc_stores'


class GmcProduct(StoreBaseModel):
  id = BigIntegerField()
  gmc_store_id = IntegerField()
  gmc_id = CharField()
  offer_id = CharField()
  expiration_date = DateTimeField()
  price = FloatField()
  currency = CharField()
  availability = CharField()
  seller_id = CharField()
  store_product_id = BigIntegerField()
  handle = CharField()

  class Meta:
    db_table = 'gmc_products'


class GmcProductStatus(StoreBaseModel):
  id = IntegerField()
  gmc_product_id = BigIntegerField()
  item_disapproved = BooleanField()
  country = CharField()
  issues = TextField()

  class Meta:
    db_table = 'gmc_product_statuses'


class GmcIssue(StoreBaseModel):
  id = IntegerField()
  code = CharField()
  servability = CharField()
  resolution = CharField()
  detail = CharField()
  documentation = CharField()

  class Meta:
    db_table = 'gmc_issues'


class GoogleAdsAccount(StoreBaseModel):
  id = IntegerField()
  impersonated_email = CharField()
  customer_id = CharField()
  service_account_data = CharField()
  active = BooleanField()

  class Meta:
    db_table = 'google_ads_accounts'


class GoogleAdsPerformance(StoreBaseModel):
  id = IntegerField()
  google_ads_account_id = IntegerField()
  merchant_id = CharField()
  item_id = CharField()
  impressions = CharField()
  clicks = IntegerField()
  ctr = FloatField()
  average_cpc = FloatField()
  orders = IntegerField()
  units_sold = IntegerField()
  revenue = FloatField()
  cost = FloatField()
  average_order_value = FloatField()
  conversions = IntegerField()
  conversions_value = FloatField()
  cost_per_conversion = FloatField()
  value_per_conversion = FloatField()
  roas = FloatField()
  all_conversions = IntegerField()
  all_conversions_value = FloatField()
  cost_per_all_conversions = FloatField()
  value_per_all_conversions = FloatField()
  search_impression_share = FloatField()
  search_click_share = FloatField()
  type = IntegerField()
  last_update_date = DateField()

  class Meta:
    db_table = 'google_ads_performances'


class GoogleAdsInsight(StoreBaseModel):
  id = IntegerField()
  google_ads_account_id = IntegerField()
  impressions = IntegerField()
  clicks = IntegerField()
  conversions = DecimalField(10, 2)
  spend = DecimalField(10, 2)
  ctr = DecimalField(10, 4)
  cpc = DecimalField(10, 4)
  cpm = DecimalField(10, 4)
  date = DateField()

  class Meta:
    db_table = 'google_ads_insights'


class BingAdsAccount(StoreBaseModel):
  id = IntegerField()
  account_id = CharField()
  customer_id = CharField()
  merchant_id = CharField()
  client_id = CharField()
  developer_token = CharField()
  refresh_token = TextField()
  active = BooleanField()

  class Meta:
    db_table = 'bing_ads_accounts'


class BingAdsInsight(StoreBaseModel):
  id = IntegerField()
  bing_ads_account_id = IntegerField()
  impressions = IntegerField()
  clicks = IntegerField()
  conversions = DecimalField(10, 2)
  spend = DecimalField(10, 2)
  ctr = DecimalField(10, 4)
  cpc = DecimalField(10, 4)
  cpm = DecimalField(10, 4)
  date = DateField()

  class Meta:
    db_table = 'bing_ads_insights'


class Order(StoreBaseModel):
  id = BigIntegerField()
  s_order_id = BigIntegerField()
  s_order_number = CharField()
  purchase_date = DateTimeField()
  total = FloatField()
  item_total = FloatField()
  tax = FloatField()
  shipping_charge = FloatField()
  discount = FloatField()
  shipping_cost = FloatField()
  product_cost = FloatField()
  refunded_at = DateTimeField()
  refund_amount = FloatField()
  cancelled_at = DateTimeField()
  cancel_reason = CharField()
  closed_at = DateTimeField()
  financial_status = CharField()
  fulfillment_status = CharField()
  note = CharField()
  order_status_url = CharField()

  class Meta:
    db_table = 'orders'


class OrderItem(StoreBaseModel):
  id = BigIntegerField()
  email = CharField()
  vendor_id = IntegerField()
  s_order_id = BigIntegerField()
  s_order_number = CharField()
  s_purchase_date = DateTimeField()

  s_total = FloatField()
  s_price = FloatField()
  s_tax = FloatField()
  s_discount = FloatField()
  s_quantity = IntegerField()

  s_line_item_id = BigIntegerField()
  s_product_id = BigIntegerField()
  s_variant_id = BigIntegerField()
  s_name = CharField()
  s_sku = CharField()
  s_url = CharField()
  s_image_url = CharField()
  s_options_text = CharField()

  first_name = CharField()
  last_name = CharField()
  address1 = CharField()
  address2 = CharField()
  company = CharField()
  city = CharField()
  state = CharField()
  zipcode = CharField()
  country = CharField()
  phone = CharField()
  billing_address = CharField()

  s_shipping_method = CharField()
  s_shipping_charge = FloatField()
  s_fulfillment_order_id = CharField()
  s_fulfillment_id = CharField()
  s_fulfillable_quantity = IntegerField()
  s_fulfillment_status = CharField()
  closed = BooleanField()
  closed_at = DateTimeField()
  s_removed = BooleanField()
  s_min_delivery_date = DateField()
  s_max_delivery_date = DateTimeField()

  s_carrier = CharField()
  s_tracking = CharField()
  s_shipping_cost = FloatField()
  s_intl_forward = BooleanField()
  s_intl_tracking = CharField()

  source = CharField()
  source_id = CharField()
  source_price = FloatField()
  fulfillable = BooleanField()
  estimate_profit = FloatField()
  profit = FloatField()

  risky = BooleanField()
  risky_action = CharField()
  risky_message = TextField()

  status = CharField()
  note = TextField()

  canceled = BooleanField()
  cancel_reason = TextField()
  cancelled_at = DateTimeField()

  refund_qty = IntegerField()
  refund_amount = FloatField()
  refund_note = CharField()
  refund_at = DateTimeField()
  refund_user_id = BigIntegerField()
  refunds = TextField()

  exported = BooleanField()
  sheet_id = CharField()
  sheet_name = CharField()

  created_at = DateTimeField()
  updated_at = DateTimeField()

  class Meta:
    db_table = 'order_items'


class FulfillOrder(StoreBaseModel):
  id = BigIntegerField()
  order_item_id = BigIntegerField()
  stc = BooleanField()
  buy_platform = CharField()
  buy_id = CharField()
  buy_order_number = CharField()
  buy_order_date = DateTimeField()
  buy_account = CharField()
  buy_price = FloatField()
  buy_quantity = IntegerField()
  buy_cost = FloatField()
  buy_carrier = CharField()
  buy_tracking = CharField()
  buy_edd = DateTimeField()
  forward_tracking = CharField()
  created_at = DateTimeField()
  updated_at = DateTimeField()

  class Meta:
    db_table = 'fulfill_orders'


class BlacklistKeyword(StoreBaseModel):
  id = BigIntegerField()
  keywords = CharField()
  match_type = CharField()
  brand = CharField()
  memo = CharField()
  company = CharField()

  class Meta:
    db_table = 'blacklist_keywords'


class ShipstationOrder(StoreBaseModel):
  id = BigIntegerField()
  order_item_id = BigIntegerField()
  order_id = BigIntegerField()
  order_number = CharField()
  order_key = CharField()
  store_id = CharField()

  class Meta:
    db_table = 'shipstation_orders'


class ShipstationShipment(StoreBaseModel):
  id = BigIntegerField()
  shipment_id = BigIntegerField()
  order_id = BigIntegerField()
  order_number = CharField()
  order_key = CharField()
  create_date = DateTimeField()
  ship_date = DateTimeField()
  shipment_cost = FloatField()
  insurance_cost = FloatField()
  tracking_number = CharField()
  carrier_code = CharField()
  service_code = CharField()
  warehouse_id = IntegerField()
  shipment_items = TextField()

  class Meta:
    db_table = 'shipstation_shipments'


class OrderNofraud(StoreBaseModel):
  id = BigIntegerField()
  s_order_id = BigIntegerField()
  s_order_number = CharField()
  transaction_id = CharField()
  decision = CharField()
  message = CharField()

  class Meta:
    db_table = 'order_nofrauds'


class OrderEvent(StoreBaseModel):
  id = BigIntegerField()
  order_item_id = BigIntegerField()
  event_type = CharField()
  event_time = DateTimeField()
  event_data = TextField()

  class Meta:
    db_table = 'order_events'


class DailyStatistic(StoreBaseModel):
  id = BigIntegerField()
  date = DateField()
  amount = DecimalField(10, 2)
  label = CharField()

  class Meta:
    db_table = 'daily_statistics'


class SellerDailyStatistic(StoreBaseModel):
  id = BigIntegerField()
  vendor_id = BigIntegerField()
  date = DateField()
  amount = DecimalField(10, 2)
  label = CharField()

  class Meta:
    db_table = 'seller_daily_statistics'


class SourceDailyStatistic(StoreBaseModel):
  id = BigIntegerField()
  source = CharField()
  date = DateField()
  amount = DecimalField(10, 2)
  label = CharField()

  class Meta:
    db_table = 'source_daily_statistics'
