import json
try:
  from urllib.parse import urlparse
  from urllib.parse import parse_qs
except ImportError:
  from urlparse import urlparse
  from urlparse import parse_qs

import pdb

import dateutil
import dateutil.parser
from shopping_utils import logger
from shopping_utils.google_shopping import GoogleShopping

from em_tasks.store.models import *


class GmcSyncProductsTask():
  def __init__(self, merchant_id):
    self.merchant_id = merchant_id
    self.gs = None
    self.gmc_store = GmcStore.get_or_none(GmcStore.merchant_id == merchant_id)
    if not self.gmc_store:
      return

    self.gs = GoogleShopping(merchant_id, json.loads(self.gmc_store.service_account_data))

  def run(self):
    if not self.gmc_store or not self.gs:
      return

    products_buf = []
    buf_size = 500
    try:
      for prod in self.gs.list_products():
        products_buf.append(prod)

        if len(products_buf) < buf_size:
          continue

        self.process_products(products_buf)
        products_buf = []

      if products_buf:
        self.process_products(products_buf)
        products_buf = []
    except Exception as e:
      logger.exception(e)

  def process_products(self, products):
    prods = dict()
    gmc_products = dict()
    gmc_ids = [product['id'] for product in products]
    for gmc_product in GmcProduct.select().where((GmcProduct.gmc_store_id == self.gmc_store.id) & (GmcProduct.gmc_id.in_(gmc_ids))).execute():
      gmc_products[gmc_product.gmc_id] = gmc_product

    to_create = []
    to_update = []
    for product in products:
      prod = {
        'gmc_store_id': self.gmc_store.id,
      }
      prod.update(self.to_gmc_product(product))
      
      if product['id'] in gmc_products:
        prod['id'] = gmc_products[product['id']].id
        to_update.append(prod)
      else:
        to_create.append(prod)

    if to_update:
      GmcProduct.insert_many(to_update).on_conflict(
        preserve=(GmcProduct.price, GmcProduct.currency, GmcProduct.availability, GmcProduct.seller_id, GmcProduct.handle)).execute()
      to_update = []

    if to_create:
      GmcProduct.insert_many(to_create).execute()
      to_create = []

    product_statuses = self.gs.get_product_statuses(gmc_ids)

    for gmc_product in GmcProduct.select().where((GmcProduct.gmc_store_id == self.gmc_store.id) & (GmcProduct.gmc_id.in_(gmc_ids))).execute():
      gmc_products[gmc_product.gmc_id] = gmc_product
    gmc_product_records = []
    for product_id, product_status in product_statuses.items():
      if product_id not in gmc_products:
        continue

      gmc_product = gmc_products[product_id]
      gmc_product_records.append({
        'id': gmc_product.id,
        'expiration_date': product_status['googleExpirationDate']
      })
    if gmc_product_records:
      GmcProduct.insert_many(gmc_product_records).on_conflict(
        preserve=(GmcProduct.expiration_date)).execute()

    gmc_issues = dict()
    for product_id, product_status in product_statuses.items():
      for gmc_issue_record in self.extract_gmc_issues(product_status):
        issue_code = gmc_issue_record['code']
        if issue_code in gmc_issues:
          continue

        gmc_issues[issue_code] = gmc_issue_record
    if gmc_issues:
      issue_codes = list(gmc_issues.keys())
      for gi in GmcIssue.select().where(GmcIssue.code.in_(issue_codes)).dicts():
        gmc_issues.pop(gi['code'], None)

      if gmc_issues:
        GmcIssue.insert_many(list(gmc_issues.values())).execute()

    gpss = dict()
    gmc_product_ids = [gmc_product.id for gmc_product in gmc_products.values()]
    for gps in GmcProductStatus.select().where(GmcProductStatus.gmc_product_id.in_(gmc_product_ids)).execute():
      k = '{}-{}'.format(gps.gmc_product_id, gps.country)
      gpss[k] = gps

    gps_to_create = []
    gps_to_update = []
    for product_id, product_status in product_statuses.items():
      if product_id not in gmc_products:
        continue

      gmc_product = gmc_products[product_id]
      approval = self.parse_approval(product_status)
      if not approval:
        continue

      item_issues = self.parse_item_issues(product_status)
      for country, item_approval in approval.items():
        issues = ''
        item_issues_c = item_issues.get(country, None)
        if item_issues_c:
          issues = json.dumps(item_issues_c)
        gps_record = {
          'gmc_product_id': gmc_product.id,
          'country': country,
          'item_disapproved': not item_approval,
          'issues': issues
        }
        k = '{}-{}'.format(gmc_product.id, country)
        if k in gpss:
          gps_record['id'] = gpss[k].id
          gps_to_update.append(gps_record)
        else:
          gps_to_create.append(gps_record)

    if gps_to_create:
      GmcProductStatus.insert_many(gps_to_create).execute()

    if gps_to_update:
      GmcProductStatus.insert_many(gps_to_update).on_conflict(
        preserve=(GmcProductStatus.item_disapproved, GmcProductStatus.issues)).execute()

  def to_gmc_product(self, product):
    url = product['link']
    parsed = urlparse(url)
    handle = parsed.path.split('/').pop()
    prod = {
      'gmc_id': product['id'],
      'offer_id': product['offerId'],
      'seller_id': product.get('externalSellerId', None),
      'handle': handle,
    }
    # Do not use "expirationDate" in product, not all products has this attribute
    # if 'expirationDate' in product:
    #   prod['expiration_date'] = product['expirationDate']

    offer = self.parse_offer(product)
    prod.update(offer)

    return prod

  def parse_offer(self, product):
    price = None
    currency = None
    price_p = product.get('price', None)
    if price_p:
      price = round(float(price_p['value']), 2)
      currency = price_p.get('currency', 'USD')

    return {
      'availability': product.get('availability', None),
      'price': price,
      'currency': currency
    }

  def parse_approval(self, prodstatus):
    approval = dict()
    if 'destinationStatuses' not in prodstatus:
      return approval

    shopping_stat = next(
      (dest_stat for dest_stat in prodstatus['destinationStatuses'] \
        if dest_stat['destination'] == 'Shopping'), None)
    if not shopping_stat:
      logger.debug('[NoShoppingDestinationStatuses] %s', prodstatus)
      return approval

    for approved_country in shopping_stat.get('approvedCountries', []):
      approval[approved_country] = True
    for disapproved_country in shopping_stat.get('disapprovedCountries', []):
      approval[disapproved_country] = False

    return approval

  def parse_item_issues(self, prodstatus):
    item_issues = dict()
    if 'itemLevelIssues' not in prodstatus:
      return item_issues

    item_level_issues = prodstatus.get('itemLevelIssues', [])
    for item_level_issue in item_level_issues:
      for country in item_level_issue['applicableCountries']:
        if country not in item_issues:
          item_issues[country] = []

        issue = dict(item_level_issue)
        issue.pop('applicableCountries', None)
        item_issues[country].append(issue)

    return item_issues

  def extract_gmc_issues(self, prodstatus):
    gmc_issues = dict()
    item_level_issues = prodstatus.get('itemLevelIssues', [])
    for item_level_issue in item_level_issues:
        gmc_issues[item_level_issue['code']] = {
          'code': item_level_issue['code'],
          'servability': item_level_issue['servability'],
          'resolution': item_level_issue['resolution'],
          'detail': item_level_issue['detail'],
          'documentation': item_level_issue['documentation']
        }

    return list(gmc_issues.values())
