# -*- coding: utf-8 -*-

import re
import datetime

from gspread.exceptions import APIError
from tracking_numbers import get_tracking_number

from em_tasks import logger
from em_tasks.store.store_util import StoreUtil
from em_tasks.google.sheet_api import SheetAPI
from em_tasks.utils.order_reader import GSheetOrderReader
from em_tasks.store.models import *


class ShopifyOrdersSheetImporter():
  def __init__(self, service_account_data, spreadsheet_ids, days):
    self.service_account_data = service_account_data
    self.sheet_api = SheetAPI(service_account_data)
    self.order_reader = GSheetOrderReader(self.sheet_api)
    self.spreadsheet_ids = spreadsheet_ids
    self.days = days
    self.tracking_pattern = re.compile('[0-9A-Za-z-]{10,100}')

  def run(self):
    for spreadsheet_id in self.spreadsheet_ids:
      for worksheet, headers, orders in self.order_reader.read(spreadsheet_id, self.days):
        records = []
        for order in orders:
          order_id = order.get('order-id', '')
          if not order_id:
            continue

          order_id_valid = True
          for part in order_id.split('-'):
            if not part.isnumeric():
              order_id_valid = False
              break
          if not order_id_valid:
            continue

          external_order_number = order.get('order number', order.get('amazon order number', '')).strip()
          external_order_cost = order.get('order cost', order.get('cost', '')).strip()
          external_order_date = order.get('order date', '')
          external_order_seller = order.get('seller', '')
          external_order_account = order.get('account', '')

          tracking = order.get('tracking number', order.get('tracking', '')).strip()
          if not self.tracking_pattern.match(tracking):
            tracking = ''

          if not (tracking or (external_order_number and external_order_cost) or external_order_date or external_order_seller):
            continue

          if external_order_cost:
            cost = None
            try:
              cost = round(float(external_order_cost), 2)
            except Exception as e:
              cost = 0
              try:
                for cost_str in external_order_cost.split("\n"):
                  cost_str = re.sub('[^0-9\.]', '', cost_str.strip())
                  cost += round(float(cost_str), 2)
              except:
                cost = None
                logger.warning('[InvalidCost] %s', order)

            try:
              price = float(order.get('price', 0))
              quantity = int(order.get('quantity', order.get('quantity-purchased', 0)))
              total_price = price * quantity
              if cost and total_price and cost > 1.5 * total_price:
                cost = None
            except:
              pass

            if cost:
              external_order_cost = cost
          else:
            external_order_cost = None

          if 'ship to customer' in order:
            ship_to_customer = order.get('ship to customer', 'Y')
            stc = ship_to_customer.lower() not in ['n', 'stw']
          else:
            remark = order.get('remark', '')
            stc = remark.lower() == 'stc'

          carrier = order.get('carrier', '')
          if not carrier:
            if tracking:
              tracking_number = get_tracking_number(tracking)
              if tracking_number and tracking_number.courier:
                carrier = tracking_number.courier.code.upper()
              else:
                carrier = 'PARCELSAPP'

          records.append({
            'order_id': order_id,
            'stc': stc,
            'buy_order_number': external_order_number,
            # 'buy_order_date': external_order_date,
            'buy_account': external_order_account or external_order_seller,
            'buy_cost': external_order_cost,
            'buy_carrier': carrier,
            'buy_tracking': tracking
          })

        if not records:
          continue

        self.process_fulfill_orders(records)

  def process_fulfill_orders(self, records):
    records_mapping = dict()
    s_line_item_ids = []
    for record in records:
      s_order_id, s_line_item_id = record['order_id'].split('-')
      s_line_item_ids.append(s_line_item_id)
      records_mapping[s_line_item_id] = record

    for order_item in OrderItem.select().where(OrderItem.s_line_item_id.in_(s_line_item_ids)).execute():
      s_line_item_id = str(order_item.s_line_item_id)
      if s_line_item_id not in records_mapping:
        continue

      record = records_mapping[s_line_item_id]
      record['order_item_id'] = order_item.id
      record.pop('order_id', None)

    fulfill_order_records = dict()
    for record in records_mapping.values():
      fulfill_order_records[record['order_item_id']] = record

    order_item_ids = list(fulfill_order_records.keys())
    for fulfill_order in FulfillOrder.select().where(FulfillOrder.order_item_id.in_(order_item_ids)).execute():
      if fulfill_order.order_item_id not in fulfill_order_records:
        continue

      is_changed = False
      fulfill_order_record = fulfill_order_records.pop(fulfill_order.order_item_id)
      for k, v in fulfill_order_record.items():
        if k == 'buy_cost':
          buy_cost = None
          if fulfill_order.buy_cost:
            buy_cost = round(fulfill_order.buy_cost, 2)

          if v != buy_cost and v:
            is_changed = True
            fulfill_order.buy_cost = v
        else:
          if getattr(fulfill_order, k) != v:
            is_changed = True
            setattr(fulfill_order, k, v)

      if is_changed:
        fulfill_order.save()

    if fulfill_order_records:
      FulfillOrder.insert_many(list(fulfill_order_records.values())).execute()

    StoreUtil.update_order_items_status(order_item_ids)
