# -*- coding: utf-8 -*-

import os
import json
import logging
import logging.handlers
import io
import pkg_resources
import datetime
import csv
import re

import click
from google.oauth2 import service_account
from google.cloud.translate_v2 import Client

from lib import logger, local_config_file_path
from lib.utils.product_service import ProductService
from lib.utils.exchange_rate import ExchangeRate


@click.command()
@click.option('-sa', '--service_account_path', type=str, help='Service account to authenticate google service.')
@click.argument('products_path')
def upload_products(service_account_path, products_path):
  if not products_path:
    logger.error('products_path is required!')
    return

  products_path = os.path.abspath(os.path.expanduser(products_path))
  if not os.path.isfile(products_path):
    logger.error('Could not find products file - %s', products_path)
    return

  if not service_account:
    logger.error("Service account is required to authenticate google service")
    return

  service_account_path = os.path.abspath(os.path.expanduser(service_account_path))
  if not os.path.isfile(service_account_path):
    logger.error("Could not find service account file - %s", service_account_path)
    return

  target_language = 'en'
  translation_service = Client(
    target_language=target_language,
    credentials=service_account.Credentials.from_service_account_file(service_account_path))

  indice_name = 'rakuten_products'

  ps = ProductService(
    '35.192.187.156', port=80, user='elasticuser', password='KbersRiseUp153')
  ps.ensure_indice(indice_name)

  products_buf = []
  batch_size = 120
  with open(products_path, encoding='utf-8', errors='ignore', newline='') as fh:
    reader = csv.reader(fh)

    headers = next(reader)
    logger.debug(headers)
    for row in reader:
      logger.debug('ColCnt: %s, Row: %s', len(row), row)

      record = dict()
      try:
        for idx, header in enumerate(headers):
          record[header] = row[idx]
      except Exception as e:
        logger.exception(e)
        record = None

      if not record:
        logger.info('NoData')
        continue

      logger.debug(record)

      products_buf.append(record)
      if len(products_buf) < batch_size:
        continue

      products = format_products(products_buf)
      titles = [product['title'] for product in products]
      result = None
      try:
        result = translation_service.translate(titles)
      except ValueError:
        pass
      except Exception as e:
        logger.exception(e)
      for idx, product in enumerate(products):
        product['_id'] = product['product_id']
        if result:
          try:
            product['title_en'] = result[idx]['translatedText']
          except:
            pass

      try:
        ps.save_products_v2(indice_name, products)
      except Exception as e:
        logger.exception(e)

      products_buf = []

    if products_buf:
      products = format_products(products_buf)
      titles = [product['title'] for product in products]
      result = None
      try:
        result = translation_service.translate(titles)
      except Exception as e:
        logger.exception(e)
      for idx, product in enumerate(products):
        product['_id'] = product['product_id']
        if result:
          product['title_en'] = result[idx]['translatedText']

      try:
        ps.save_products_v2(indice_name, products)
      except Exception as e:
        logger.exception(e)

      products_buf = []

def format_products(prods):
  products = []
  price_pattern = re.compile('([\d.]+)')
  now = datetime.datetime.now().isoformat(' ')
  exchange_rate = ExchangeRate.get_exchange_rate('USD', 'JPY')
  for prod in prods:
    product_url = prod['product_url'].strip()
    if not product_url:
      logger.debug('[NoProductUrl] %s', prod)
      continue

    product_id = product_url.split('?').pop(0).split('rakuten.co.jp').pop()
    # product_id = product_url.split('?').pop(0).split('rakuten.co.jp').pop().strip('/').replace('/', '#')
    if not product_id:
      logger.debug('[NoProductID] %s', prod)
      continue

    if prod['variations'] and prod['variations'].find('variant_id') != -1:
      logger.debug('[SkipProductWithVariation] %s', prod)
      continue

    if not prod['price']:
      logger.debug('[NoProductPrice] %s', prod)
      continue

    price = None
    match = re.search(price_pattern, prod['price'].replace(',', ''))
    if match:
      price = round(float(match.group(1)), 2)
    if not price:
      logger.debug('[ParseProductPriceError] %s', prod)
      continue

    price_in_usd = round(price / exchange_rate, 2)
    if price_in_usd:
      qty = None
    else:
      qty = 0

    images = prod['images']
    if not images:
      logger.debug('[NoProductImage] %s', prod)
      continue

    images = ';'.join(prod['images'].split(','))

    cats = [cat.strip() for cat in prod['category'].split('>') if cat.strip()]
    if cats:
      categories = cats[0]
    else:
      categories = 'Beauty & Health'

    reviews = None
    if prod['reviews count']:
      try:
        reviews = int(prod['reviews count'])
      except:
        pass

    rating = None
    if prod['rating']:
      try:
        rating = round(float(prod['rating']), 2)
      except:
        pass

    products.append({
      'date': now,
      'url': prod['product_url'],
      'source': 'rakuten',
      'product_id': product_id,
      'existence': True,
      'title': prod['title'],
      'description': prod['description'],
      'summary': None,
      'sku': 'Rakuten_{}'.format(product_id),
      'upc': None,
      'brand': None,
      'categories': categories,
      'images': images,
      'videos': None,
      'price': price_in_usd,
      'available_qty': qty,
      'options': None,
      'variants': None,
      'returnable': False,
      'reviews': reviews,
      'rating': rating,
      'sold_count': None,
      'shipping_fee': 1,
      'shipping_days_min': None,
      'shipping_days_max': None,
      'weight': None,
      'width': None,
      'height': None,
      'length': None
    })

  return products


if __name__ == '__main__':
  upload_products()
