import time
import sys
import json

from elasticsearch import Elasticsearch
from elasticsearch import helpers
from elasticsearch.client import IndicesClient
from elasticsearch.exceptions import RequestError
from elasticsearch.exceptions import NotFoundError
from elasticsearch.exceptions import ConnectionTimeout
from elasticsearch.exceptions import ConnectionError
from elasticsearch.exceptions import SSLError
from elasticsearch.exceptions import TransportError
from elasticsearch.exceptions import ElasticsearchException

from dropshipping.utils.es_service import es_retry

from em_tasks import logger


class ProductService(object):
  def __init__(self, host, port, user, password, max_retry=3):
    self.host = host
    self.port = port
    self.user = user
    self.password = password
    self.max_retry = max_retry

    self.esclient = Elasticsearch(hosts=host, port=port, http_auth=(user, password))

  def ensure_indice(self, indice_name, settings=None):
    ic = IndicesClient(self.esclient)
    if ic.exists(indice_name):
      return True

    if not settings:
      settings = {"index.mapping.total_fields.limit": 100000}
    ic.create(indice_name, {'settings': settings})
    return ic.exists(indice_name)

  def search_products(self, indice_name, product_ids, options={}):
    if not isinstance(product_ids, list):
      product_ids = [product_ids]

    query = {"terms": {"_id": product_ids}}

    params = {
      'index': indice_name,
      'from_': 0,
      'size': len(product_ids),
      'body': {
        'query': query
      }
    }
    if options:
        params.update(options)

    wrapped_search = es_retry(self.esclient.search)
    resp = None
    try:
      resp = wrapped_search(**params)
    except:
      pass

    if resp is None:
      result = None
    elif resp == -1:
      result = False
    else:
      result = {}
      for item in resp['hits']['hits']:
        result[item['_id']] = item['_source']

    return result

  def custom_search(self, indice_name, query, from_=0, size=1000):
    params = {
      'index': indice_name,
      'from_': from_,
      'size': size,
      'body': {
        'query': query
      }
    }

    wrapped_search = es_retry(self.esclient.search)
    resp = None
    try:
      resp = wrapped_search(**params)
    except Exception as e:
      logger.exception(e)

    return resp

  def save_products(self, indice_name, products):
      """
      Save web crawled products to service.
      products : list
          The same as return value of search_products
      """
      if not isinstance(products, list):
        products = [products]

      service_products = []

      common_args = {
        '_op_type': 'index',
        '_index': indice_name,
        '_type': '_doc'
      }

      for product in products:
        service_product = dict()
        service_product.update(common_args)
        if 'product_id' not in product and '_id' not in product:
          continue

        if 'product_id' in product:
          service_product['_id'] = product['product_id']
        else:
          service_product['_id'] = product.pop('_id')

        service_product['_source'] = product

        service_products.append(service_product)

      retry = self.max_retry
      while retry > 0:
        try:
          helpers.bulk(self.esclient, service_products, request_timeout=30)
          break
        except (ConnectionTimeout, ConnectionError, SSLError, TransportError):
          retry -= 1
          continue
        except Exception as e:
          raise e

  def load_products(self, indice_name, options = {}):
    params = {
      'index': indice_name,
      'doc_type': '_doc',
      'size': 1500,
      'query': {'query': {'match_all': {}}}
    }
    if options:
      params.update(options)

    wrapped_scan = es_retry(helpers.scan)
    try:
      for item in wrapped_scan(self.esclient, **params):
        if isinstance(item['_source'], dict):
          record = item['_source']
        else:
          record = json.loads(item['_source'])

        yield (item['_id'], record)
    except Exception as e:
      logger.exception(e)

  def delete_products(self, indice_name, product_ids):
    actions = []
    for product_id in product_ids:
      actions.append({
        '_op_type': 'delete',
        '_index': indice_name,
        '_type': '_doc',
        '_id': product_id
      })

    retry = self.max_retry
    while retry > 0:
      try:
        helpers.bulk(self.esclient, actions, request_timeout=30)
        break
      except (ConnectionTimeout, ConnectionError, SSLError, TransportError):
        retry -= 1
        continue
      except Exception as e:
        raise e
